var swipe = {
    types: Object.freeze({"horizontal":1,"vertical":2,"vector":3}),
    swipes: [], // store for elements and swipe options
    addSwipe: function(elemId, params){
        var opts = this._setOptions(params);
        opts.elemId = elemId;
        let el = document.getElementById(elemId);
        let ts = this; // as this is not the same this in the following 3 lines
        el.addEventListener("pointerdown", function(e) {ts._start(e);}, false);
        el.addEventListener("pointermove", function(e) {ts._move(e);}, false);
        el.addEventListener("pointerup", function(e) {ts._end(e);}, false);
        this.swipes.push(opts);
    },
    _start: function(e) {
        for(let s = 0; s < this.swipes.length; s++){
            if(this.swipes[s].elemId === e.currentTarget.id){
                if(this.swipes[s].eventsDisabled){
                    return;
                }
                if(this.swipes[s].preventDefault){
                    e.preventDefault();
                }
                this.swipes[s].startTime = e.timeStamp;
                this.swipes[s].startX = e.clientX;
                this.swipes[s].startY = e.clientY;
                this.swipes[s].lastMove = 0;
                break;
            }
        }
    },
    _move: function(e){
        for(let s = 0; s < this.swipes.length; s++){
            if(this.swipes[s].elemId === e.currentTarget.id){
                if(this.swipes[s].eventsDisabled){
                    return;
                }
                if(this.swipes[s].preventDefault){
                    e.preventDefault();
                }
                let dx = e.clientX - this.swipes[s].startX;
                let dy = e.clientY - this.swipes[s].startY;
                let distance, direction, retDistance;
                retDistance = distance = Math.hypot(dx, dy);
                let currentTime = e.timeStamp - this.swipes[s].startTime;
                switch(this.swipes[s].type){
                    case this.types.horizontal:
                        direction = (dx > 0) ? "right" : "left";
                        break;
                    case this.types.vertical:
                        direction = (dy > 0) ? "down" : "up";
                        break;
                    case this.types.vector:
                        direction = this._posRad(Math.atan2(dy, dx)).toFixed(4);
                        break;
                }
                if(this.swipes[s].incrementsOnly){
                    retDistance = distance - this.swipes[s].lastMove;
                    this.swipes[s].lastMove = distance;
                }
                if(distance >= this.swipes[s].minDistance && currentTime <= this.swipes[s].maxTime){
                    retDistance = Math.round(retDistance);
                    this.swipes[s].onSwipe(this.swipes[s].elemId, direction,
                        retDistance, Math.round((distance / currentTime) * 1000));
                }
                break;
            }
        }
    },
    _end: function(e) {
        for(let s = 0; s < this.swipes.length; s++){
            if(this.swipes[s].elemId === e.currentTarget.id){
                if(this.swipes[s].swipeEnd){
                    let currentTime = e.timeStamp - this.swipes[s].startTime;
                    let dx = e.clientX - this.swipes[s].startX;
                    let dy = e.clientY - this.swipes[s].startY;
                    let direction;
                    switch(this.swipes[s].type){
                        case this.types.horizontal:
                            direction = (dx > 0) ? "right" : "left";
                            break;
                        case this.types.vertical:
                            direction = (dy > 0) ? "down" : "up";
                            break;
                        case this.types.vector:
                            direction = this._posRad(Math.atan2(dy, dx)).toFixed(4);
                            break;
                    }                       
                    this.swipes[s].swipeEnd(this.swipes[s].elemId, direction,
                        Math.round(this.swipes[s].lastMove), 
                        Math.round((this.swipes[s].lastMove / currentTime) * 1000));
                }
                break;
            }
        }
    },
    _setOptions: function(params){
        var opts = {
            eventsDisabled: false,
            maxTime: 1000,
            minDistance: 10, // aprox 2.5mm+
            preventDefault: true,
            type: swipe.types.horizontal,
            onSwipe: function(id, direction, howfar, speed){},
            swipeEnd: null, // optional function
            incrementsOnly: true, // only pass incremental distances
            startX: 0,
            startY: 0,
            lastMove: 0,
            startTime: 0,
            elemId: ''
        };
        let opt, defopts;
        for(opt in params){
            for (defopts in opts){
                if(opt === defopts){
                    opts[opt] = params[opt];
                    break;
                }
            }
        }
        return opts;
    },
    _posRad: function(angle){
            const twoPi = Math.PI * 2;
            angle %= twoPi;
            while(angle < 0) {angle += twoPi;}
            return angle;
    },
    disable: function(elem){
        for(let s = 0; s < this.swipes.length; s++){
            if(this.swipes[s].elemId === elem){
                this.swipes[s].eventsDisabled = true;
            }
        }
    },
    enable: function(elem) {
        for(let s = 0; s < this.swipes.length; s++){
            if(this.swipes[s].elemId === elem){
                this.swipes[s].eventsDisabled = false;
            }
        }
    }
};